function integral = NavierStokesContinuity(u, v, w, du, dv, dw, dx, ds, x, d, t, eq)

% Variational formulation of the continuity equation in the Navier-Stokes system.
% (c) Johan Hoffman and Anders Logg 2003.
% Licensed under the GNU GPL Version 2.

d1 = 2*sqrt(dx/pi);

% w(1)  = previous time value of u1: up1
% w(2)  = previous time value of u2: up2
% w(3)  = frozen velocity field U1
% w(4)  = frozen velocity field U2
% w(5)  = pressure p 

U   = [w(3); w(4)];
dU1 = dw(:,3);
dU2 = dw(:,4);

if eq == 1
  integral = d1*du'*dv*dx + 0.1*u*v*dx;
else
  integral = (- d1*((U'*dU1)*dv(1)+(U'*dU2)*dv(2)) - (dU1(1)+dU2(2))*v + d1*f(x,d,t)'*dv)*dx;
end

%--- Right-hand side, source term ---
function y = f(x, d, t)
y = [0;0];
