function integral = NavierStokesMomentum1(u, v, w, du, dv, dw, dx, ds, x, d, t, eq)

% Variational formulation of the first momentum equation in the Navier-Stokes system.
% (c) Johan Hoffman and Anders Logg 2003.
% Licensed under the GNU GPL Version 2.

k  = 0.05;
a  = 0.01;
d1 = 2*sqrt(dx/pi);

% w(1)  = previous time value of u1: up1
% w(2)  = previous time value of u2: up2
% w(3)  = frozen velocity field U1
% w(4)  = frozen velocity field U2
% w(5)  = pressure p 

Up1 = w(1);
Up2 = w(2);
Px  = dw(1,5);
Py  = dw(2,5);
U   = [w(3); w(4)];

if eq == 1
  integral = u*v*dx + k*(a*du'*dv*dx + (U'*du)*(v + d1*(U'*dv))*dx + g(x,d,t)*u*v*ds);
else
  integral = k*((f1(x,d,t) - Px)*(v + d1*(U'*dv))*dx + (g(x,d,t)*gd(x,d,t) - gn(x,d,t))*v*ds) + Up1*v*dx;
end

%--- Conductivity (penalty factor) ---
function y = g(x, d, t)

% Flow around cylinder
if d == 3 | d == 5
  y = 1e7;
else
  y = 0;
end  

% Driven cavity
%y = 1e7;  

%--- Dirichlet boundary condition ----
function y = gd(x, d, t)

% Flow around cylinder
if d == 3
  y = 1;
else
  y = 0;
end  

% Driven cavity
%if abs(x(2)-1) < 1e-2
%  y = 1;
%else 
%  y = 0;
%end

%--- Neumann boundary condition ---
function y = gn(x, d, t)
y = 0;

%--- Right-hand side, source term ---
function y = f1(x, d, t)
y = 0;
