% A simple solver for Navier-Stokes equations (cG(1)dG(0). 
% (c) Johan Hoffman and Anders Logg 2003.
% Licensed under the GNU GPL Version 2.

% Load the mesh
%square
cylinder

T = 20;
k = 0.05;
time = 0;
N = floor(T/k);

% Initial condition
Uxp = zeros(size(p,2),1);
Uyp = zeros(size(p,2),1);

Ux = zeros(size(p,2),1);
Uy = zeros(size(p,2),1);
P  = zeros(size(p,2),1);

Ux0 = zeros(size(p,2),1);
Uy0 = zeros(size(p,2),1);
P0  = zeros(size(p,2),1);

% Coefficients
% w(1)  = previous time value of u1
% w(2)  = previous time value of u2
% w(3)  = frozen velocity field U1
% w(4)  = frozen velocity field U2
% w(5)  = pressure p 

clear M

% Time-stepping
for n = 1:N
     
  time = time + k;
  Uxp = Ux;
  Uyp = Uy;
	
  % Non linear loop 
  for i = 1:10 

    Ux0 = Ux;
    Uy0 = Uy;
    P0  = P;
    
    % Assemble matrix
    A1 = AssembleMatrix(p, e, t, 'NavierStokesMomentum1',  [Uxp, Uyp, Ux, Uy, P], time);
    b1 = AssembleVector(p, e, t, 'NavierStokesMomentum1',  [Uxp, Uyp, Ux, Uy, P], time);

    % Solve the linear system
    Ux = A1 \ b1;
   
    %disp(['norm A1 = ' num2str(norm(A1))])
    %disp(['norm b1 = ' num2str(norm(b1))])
    %disp(['norm Ux = ' num2str(norm(Ux))])
  
    % Assemble
    A2 = AssembleMatrix(p, e, t, 'NavierStokesMomentum2',  [Uxp, Uyp, Ux, Uy, P], time);
    b2 = AssembleVector(p, e, t, 'NavierStokesMomentum2',  [Uxp, Uyp, Ux, Uy, P], time);

    % Solve the linear system
    Uy = A2 \ b2;
   
    %disp(['norm A2 = ' num2str(norm(A2))])
    %disp(['norm b2 = ' num2str(norm(b2))])
    %disp(['norm Uy = ' num2str(norm(Uy))])

    % Assemble matrix
    A3 = AssembleMatrix(p, e, t, 'NavierStokesContinuity', [Uxp, Uyp, Ux, Uy, P], time);
    b3 = AssembleVector(p, e, t, 'NavierStokesContinuity', [Uxp, Uyp, Ux, Uy, P], time);

    % Solve the linear system
    P = A3 \ b3;

    %disp(['norm A3 = ' num2str(norm(A3))])
    %disp(['norm b3 = ' num2str(norm(b3))])
    %disp(['norm P = ' num2str(norm(P))])

    nlin_res = norm([Ux, Uy,P]-[Ux0,Uy0,P0]);
    disp(['norm of non linear residual = ' num2str(nlin_res)])
 
    if (nlin_res < 0.02)
      break;
    end
  
  end 
   
  % Plot solution
  %pdesurf(p, t, sqrt((Ux).^2+(Uy).^2))
  %shading interp
  quiver(p(1,:),p(2,:),Ux,Uy)
  view(2)
  %M(n) = getframe;
  drawnow
  disp(['t = ' num2str(time)])
%  disp(['t = ' num2str(time) ' Press any key to continue.'])
%  pause
	
  if time >= T
    break;
  end

end

%movie(M)
